<?php
namespace WeChat;

class JSSDK
{
    private $appId;
    private $appSecret;

    public function __construct($appId, $appSecret)
    {
        $this->appId     = $appId;
        $this->appSecret = $appSecret;
    }

    public function getSignPackage()
    {
        $jsapiTicket = $this->getJsApiTicket();

        // 注意 是当前URL地址 如果不是动态生成的js引用的话直接用下面注释的代码就可以，否则用下面来源地址
        // $protocol = (!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] !== 'off' || $_SERVER['SERVER_PORT'] == 443) ? "https://" : "http://";
        // $url      = "$protocol$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]";
        $url = isset($_GET['refer']) ? $_GET['refer'] : '';
        if (!$url) {
            $url = isset($_SERVER['HTTP_REFERER']) ? $_SERVER['HTTP_REFERER'] : '';
        }
        //$url       = 'http://zhushou.ngrok.bxd365.com/home/default';
        $timestamp = time();
        $nonceStr  = $this->createNonceStr();

        // 这里参数的顺序要按照 key 值 ASCII 码升序排序
        $string                = "jsapi_ticket=$jsapiTicket&noncestr=$nonceStr&timestamp=$timestamp&url=$url";
        $_SESSION['sg_string'] = $string;
        // $this->set_php_file('data.log', $string);
        $signature = sha1($string);

        $signPackage = array(
            "appId"     => $this->appId,
            "nonceStr"  => $nonceStr,
            "timestamp" => $timestamp,
            "url"       => $url,
            "signature" => $signature,
            "rawString" => $string,
        );
        return $signPackage;
    }

    private function createNonceStr($length = 16)
    {
        $chars = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
        $str   = "";
        for ($i = 0; $i < $length; $i++) {
            $str .= substr($chars, mt_rand(0, strlen($chars) - 1), 1);
        }
        return $str;
    }

    private function getJsApiTicket()
    {
        // jsapi_ticket 应该全局存储与更新，以下代码以写入到文件中做示例
        // $data = json_decode($this->get_php_file("jsapi_ticket.php"), true);
        $data = [];
        if (isset($_SESSION['jsapi_ticket'])) {
            $data = $_SESSION['jsapi_ticket'];
        }
        // $data = F('jsapi_ticket');
        $accesstoken = '';
        if (!$data || $data['expire_time'] < time()) {
            $accessToken = $this->getAccessToken();
            // 如果是企业号用以下 URL 获取 ticket
            // $url = "https://qyapi.weixin.qq.com/cgi-bin/get_jsapi_ticket?access_token=$accessToken";
            $url = "https://api.weixin.qq.com/cgi-bin/ticket/getticket?type=jsapi&access_token=$accessToken";
            $res = json_decode($this->httpGet($url));
            if (!$res) {
                echo 'get JsApiTicket error!';
            }
            $ticket = $res->ticket;
            if ($ticket) {
                $data['expire_time']  = time() + 7200;
                $data['jsapi_ticket'] = $ticket;
                // $this->set_php_file("jsapi_ticket.php", json_encode($data));
                $_SESSION['jsapi_ticket'] = $data;
                // F('jsapi_ticket', $data);
            }
        } else {
            $ticket = $data['jsapi_ticket'];
        }

        return $ticket;
    }

    private function getAccessToken()
    {
        // access_token 应该全局存储与更新，以下代码以写入到文件中做示例
        // $data         = json_decode($this->get_php_file("access_token.php"), true);
        $data = [];
        if (isset($_SESSION['access_token'])) {
            $data = $_SESSION['access_token'];
        }
        $access_token = '';
        if (!$data || $data['expire_time'] < time()) {
            // 如果是企业号用以下URL获取access_token
            // $url = "https://qyapi.weixin.qq.com/cgi-bin/gettoken?corpid=$this->appId&corpsecret=$this->appSecret";
            $url = "https://api.weixin.qq.com/cgi-bin/token?grant_type=client_credential&appid=$this->appId&secret=$this->appSecret";
            $res = json_decode($this->httpGet($url));
            if (!$res) {
                echo 'get accesstoken error!';
            }
            $access_token = $res->access_token;
            if ($access_token) {
                $data                 = [];
                $data['expire_time']  = time() + 7200;
                $data['access_token'] = $access_token;
                // $this->set_php_file("access_token.php", json_encode($data));
                $_SESSION['access_token'] = $data;
                // F('access_token', $data);
            }
        } else {
            $access_token = $data['access_token'];
        }
        return $access_token;
    }

    private function httpGet($url)
    {
        return file_get_contents($url);
        //用下面取不到值,原因未知
        // $curl = curl_init();
        // curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        // curl_setopt($curl, CURLOPT_TIMEOUT, 500);
        // // 为保证第三方服务器与微信服务器之间数据传输的安全性，所有微信接口采用https方式调用，必须使用下面2行代码打开ssl安全校验。
        // // 如果在部署过程中代码在此处验证失败，请到 http://curl.haxx.se/ca/cacert.pem 下载新的证书判别文件。
        // curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, true);
        // curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, 2);
        // curl_setopt($curl, CURLOPT_URL, $url);

        // $res = curl_exec($curl);
        // curl_close($curl);
        // return $res;
    }
    private function getDataDir()
    {
        $dirpath = __DIR__ . '/data';
        if (!file_exists($dirpath)) {
            mkdir($dirpath, 777, true);
        }
        return $dirpath;
    }
    private function get_php_file($filename)
    {
        $dirpath = $this->getDataDir();
        return trim(substr(file_get_contents($dirpath . '/' . $filename), 15));
    }
    private function set_php_file($filename, $content)
    {
        $dirpath = $this->getDataDir();
        file_put_contents($dirpath . '/' . $filename, "<?php exit();?>" . $content);
    }
}
